dframe.check.fcn <- function(nresp, siteID, sites, subpop, data.cat, type.cat,
   data.cont, design.names) {


# Function:   dframe.check.fcn
# Programmer: Tom Kincaid
# Date:       March 11, 2004
# Description:
#   This function checks site IDs, the sites data frame, the subpop data
#      frame, the data.cat data frame, type.cat, and the data.cont data frame
#      to assure valid contents.  If they do not exist, then the sites data 
#      frame, the subpop data frame, and type.cat are created.
#   Input:
#      nresp = the number of response values.
#      siteID = site IDs.
#      sites = the sites data frame.
#      subpop = the subpop data frame.
#      data.cat = the data.cat data frame of categorical response variables.
#      type.cat = the type of categorical response variables in the data.cat
#         data frame.
#      data.cont = the data.cont data frame of continuous response variables.
#      design.names = names for the design data frame.
#   Output:
#      A list consisting of the sites data frame, the subpop data frame, the
#      data.cat data frame, type.cat, and the data.cont data frame.
#   Other Functions Required:
#      vecprint.fcn - takes an input vector and outputs a character string with
#         line breaks inserted
#      uniqueID.fcn - creates unique site IDs by appending a unique number to
#         each occurrence of a site ID

# Check site IDs for repeat values

   temp <- sapply(split(siteID, siteID), length)
   if(any(temp > 1)) {
      temp.str <- vecprint.fcn(names(temp)[temp > 1])
      warning(paste("\nThe following site ID values occur more than once among the values that were \ninput to the function:\n", temp.str, "\n", sep=""))
      siteID.t <- uniqueID.fcn(siteID)
   } else {
      siteID.t <- siteID
   }

# Check the sites data frame for contents

   if(is.null(sites)) {
      sites <- data.frame(siteID=siteID, Use.Sites=rep(TRUE, nresp))
   } else {
      if(!is.data.frame(sites))
         stop("\nThe sites argument must be a data frame.")
      temp <- dim(sites)[2]
      if(temp != 2)
         stop("\nThe sites argument must contain exactly two variables.")
      temp <- sapply(split(sites[,1], sites[,1]), length)
      if(any(temp > 1)) {
         temp.str <- vecprint.fcn(names(temp)[temp > 1])
         warning(paste("\nThe following site ID values occur more than once in the sites data frame:\n", temp.str, "\n", sep=""))
         tempID <- uniqueID.fcn(sites[,1])
      } else {
         tempID <- sites[,1]
      }
      temp <- match(siteID.t, tempID, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(siteID[temp == 0])
         stop(paste("\nThe following site ID values that were input to the function do not occur among \nthe site ID values in the sites data frame:\n", temp.str, "\n", sep=""))
      }
      temp <- match(tempID, siteID.t, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(sites[,1][temp == 0])
         stop(paste("\nThe following site ID values in the sites data frame do not occur among the \nsite ID values that were input to the function:\n", temp.str, "\n", sep=""))
      }
      sites <- sites[temp,]
      if(!is.logical(sites[,2]))
         stop("\nThe second variable in the sites data frame was not a logical variable.")
   }
   names(sites)[1] <- design.names[1]

# Check the subpop data frame for contents

   if(is.null(subpop)) {
      subpop <- data.frame(siteID=siteID, All.Sites=factor(rep("All Sites", nresp)))
   } else {
      if(!is.data.frame(subpop))
         stop("\nThe subpop argument must be a data frame.")
      temp <- dim(subpop)[2]
      if(temp < 2)
         stop("\nThe subpop argument must contain at least two variables.")
      temp <- sapply(split(subpop[,1], subpop[,1]), length)
      if(any(temp > 1)) {
         temp.str <- vecprint.fcn(names(temp)[temp > 1])
         warning(paste("\nThe following site ID values occur more than once in the subpop data frame:\n", temp.str, "\n", sep=""))
         tempID <- uniqueID.fcn(subpop[,1])
      } else {
         tempID <- subpop[,1]
      }
      temp <- match(siteID.t, tempID, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(siteID[temp == 0])
         stop(paste("\nThe following site ID values that were input to the function do not occur among \nthe site ID values in the subpop data frame:\n", temp.str, "\n", sep=""))
      }
      temp <- match(tempID, siteID.t, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(subpop[,1][temp == 0])
         stop(paste("\nThe following site ID values in the subpop data frame do not occur among the \nsite ID values that were input to the function:\n", temp.str, "\n", sep=""))
      }
      subpop <- subpop[temp,]
   }
   names(subpop)[1] <- design.names[1]

# Check the data.cat data frame for contents

   if(!is.null(data.cat)) {
      if(!is.data.frame(data.cat))
         stop("\nThe data.cat argument must be a data frame.")
      temp <- sapply(split(data.cat[,1], data.cat[,1]), length)
      if(any(temp > 1)) {
         temp.str <- vecprint.fcn(names(temp)[temp > 1])
         warning(paste("\nThe following site ID values occur more than once in the data.cat data frame:\n", temp.str, "\n", sep=""))
         tempID <- uniqueID.fcn(data.cat[,1])
      } else {
         tempID <- data.cat[,1]
      }
      temp <- match(siteID.t, tempID, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(siteID[temp == 0])
         stop(paste("\nThe following site ID values that were input to the function do not occur among \nthe site ID values in the data.cat data frame:\n", temp.str, "\n", sep=""))
      }
      temp <- match(tempID, siteID.t, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(data.cat[,1][temp == 0])
         stop(paste("\nThe following site ID values in the data.cat data frame do not occur among the \nsite ID values that were input to the function:\n", temp.str, "\n", sep=""))
      }
      data.cat <- data.cat[temp,]
      names(data.cat)[1] <- design.names[1]
   }

# Check type.cat for agreement with the data.cat data frame

   if(!is.null(data.cat)) {
      names.catvar <- names(data.cat)[-1]
      if(is.null(type.cat)) {
         type.cat <- rep("Category", dim(data.cat)[2] - 1)
         names(type.cat) <- names.catvar
      } else {
         if(is.null(names(type.cat)))
            stop("\nThe vector of variable type (Category or Status) for each categorical response \nvariable in the data.cat data frame must be named")
         names.type <- names(type.cat)
         temp <- match(names.type, names.catvar, nomatch=0)
         if(any(temp == 0)) {
            temp.str <- vecprint.fcn(names.type[temp == 0])
            stop(paste("\nThe following values in the vector of variable type (Category or Status) do not \noccur among the categorical response variables in the data.cat data frame:\n", temp.str, "\n", sep=""))
         }
         temp <- match(names.catvar, names.type, nomatch=0)
         if(any(temp == 0)) {
            temp.str <- vecprint.fcn(names.catvar[temp == 0])
            stop(paste("\nThe following categorical response variables in the data.cat data frame do not \noccur among the values in the vector of variable type (Category or Status):\n", temp.str, "\n", sep=""))
         }
         type.cat <- type.cat[temp]
      }
   }

# Check the data.cont data frame for contents

   if(!is.null(data.cont)) {
      if(!is.data.frame(data.cont))
         stop("\nThe data.cont argument must be a data frame.")
      temp <- sapply(split(data.cont[,1], data.cont[,1]), length)
      if(any(temp > 1)) {
         temp.str <- vecprint.fcn(names(temp)[temp > 1])
         warning(paste("\nThe following site ID values occur more than once in the data.cont data frame:\n", temp.str, "\n", sep=""))
         tempID <- uniqueID.fcn(data.cont[,1])
      } else {
         tempID <- data.cont[,1]
      }
      temp <- match(siteID.t, tempID, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(siteID[temp == 0])
         stop(paste("\nThe following site ID values that were input to the function do not occur among \nthe site ID values in the data.cont data frame:\n", temp.str, "\n", sep=""))
      }
      temp <- match(tempID, siteID.t, nomatch=0)
      if(any(temp == 0)) {
         temp.str <- vecprint.fcn(data.cont[,1][temp == 0])
         stop(paste("\nThe following site ID values in the data.cont data frame do not occur among the \nsite ID values that were input to the function:\n", temp.str, "n", sep=""))
      }
      data.cont <- data.cont[temp,]
      names(data.cont)[1] <- design.names[1]
   }

# Return the list

   list(sites=sites, subpop=subpop, data.cat=data.cat, type.cat=type.cat,
      data.cont=data.cont)
}
